@@ -89,6 +89,14 @@ oobs_share_nfs_finalize (GObject *object)
     (* G_OBJECT_CLASS (oobs_share_nfs_parent_class)->finalize) (object);
 }
 
+/**
+ * oobs_share_nfs_new:
+ * @path: share path.
+ * 
+ * Returns a new NFS share for the given path.
+ * 
+ * Return Value: A newly allocated #OobsShareNFS.
+ **/
 OobsShare*
 oobs_share_nfs_new (const gchar *path)
 {
@@ -97,6 +105,15 @@ oobs_share_nfs_new (const gchar *path)
 		       NULL);
 }
 
+/**
+ * oobs_share_nfs_add_acl_element:
+ * @share: An #OobsShareNFS.
+ * @element: Host in the share ACL.
+ * @read_only: Whether the share is read only for the element.
+ * 
+ * Adds an ACL entry for a host, @element may be a host name,
+ * an IP address or a combination in the form "IP address/Network mask".
+ **/
 void
 oobs_share_nfs_add_acl_element (OobsShareNFS *share,
 			       const gchar *element,
@@ -116,6 +133,15 @@ oobs_share_nfs_add_acl_element (OobsShareNFS *share,
   priv->acl = g_slist_append (priv->acl, elem);
 }
 
+/**
+ * oobs_share_nfs_set_acl:
+ * @share: An #OobsShareNFS.
+ * @acl: A #GList of #OobsShareAclElement.
+ * 
+ * Overwrites the list of ACL entries for the share. The previous list and its
+ * contents will be free, so any merging will have to be done by hand.
+ * Alternatively, you can use oobs_share_nfs_add_acl_element().
+ **/
 void
 oobs_share_nfs_set_acl (OobsShareNFS *share, GSList *acl)
 {
@@ -132,6 +158,15 @@ oobs_share_nfs_set_acl (OobsShareNFS *share, GSList *acl)
   priv->acl = acl;
 }
 
+/**
+ * oobs_share_nfs_get_acl:
+ * @share: An #OobsShareNFS.
+ * 
+ * Returns the ACL defined for this share.
+ * 
+ * Return Value: A #GList containing #OobsShareAclElement structs,
+ *               this list must be freed with g_list_free().
+ **/
 GSList*
 oobs_share_nfs_get_acl (OobsShareNFS *share)
 {
@@ -141,5 +176,5 @@ oobs_share_nfs_get_acl (OobsShareNFS *share)
   g_return_val_if_fail (OOBS_IS_SHARE_NFS (share), NULL);
   priv = share->_priv;
 
-  return priv->acl;
+  return g_list_copy (priv->acl);
 }
